//+------------------------------------------------------------------+
//|                                                 Wilder's DMI.mq4 |
//|                                                  coded by mladen |
//|                                                                  |
//| Directional movement index was developed by Welles Wilder        |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property  indicator_buffers   5
#property indicator_color1     clrLimeGreen
#property indicator_color2     clrRed
#property indicator_color3     clrDodgerBlue
#property indicator_color4     clrSandyBrown
#property indicator_color5     clrSandyBrown
#property indicator_style1     STYLE_DOT
#property indicator_style2     STYLE_DOT
#property indicator_width3     2
#property indicator_width4     2
#property indicator_width5     2
#property strict

//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted    // Weighted
};

extern ENUM_TIMEFRAMES    TimeFrame     = PERIOD_CURRENT;   // Time frame
extern int                AdxPeriod     = 14;               // Adx period
extern int                Smooth        = 5;                // Adx price smooth period
extern ENUM_MA_METHOD     SmoothMethod  = MODE_EMA;         // Adx price smooth ma method
extern ENUM_APPLIED_PRICE PriceHigh     = PRICE_HIGH;       // Adx high price to use
extern ENUM_APPLIED_PRICE PriceLow      = PRICE_LOW;        // Adx low price to use
extern ENUM_APPLIED_PRICE PriceClose    = PRICE_CLOSE;      // Adx close price to use
extern double             Level         = 20.0;             // Adx level
extern bool               Interpolate   = true;             // Interpolate in mtf mode

double DIp[],DIm[],ADX[],ADXda[],ADXdb[],trend[],count[];
string indicatorFileName;
#define _mtfCall(_buff,_ind) iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,AdxPeriod,Smooth,SmoothMethod,PriceHigh,PriceLow,PriceClose,Level,_buff,_ind)

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int init()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0,DIp);    SetIndexLabel(0,"DI+");
   SetIndexBuffer(1,DIm);    SetIndexLabel(1,"DI-");
   SetIndexBuffer(2,ADX);    SetIndexLabel(2,"ADX");
   SetIndexBuffer(3,ADXda);  SetIndexLabel(3,"ADX");
   SetIndexBuffer(4,ADXdb);  SetIndexLabel(4,"ADX");
   SetIndexBuffer(5,trend);
   SetIndexBuffer(6,count);
         for (int i=0;i<3;i++) SetIndexEmptyValue(i,0.00);
        
   indicatorFileName = WindowExpertName();
   TimeFrame         = fmax(TimeFrame,_Period);
   SetLevelValue(0,Level);
         
   IndicatorShortName(timeFrameToString(TimeFrame)+" Wilder ADX ("+(string)AdxPeriod+")");
return(0);
}
int deinit() { return(0);}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double work[][3];
#define _DIp 0
#define _DIm 1
#define _TR  2

int start()
{
   int i,r,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = fmin(Bars-counted_bars,Bars-1); count[0] = limit;
         if (TimeFrame != _Period)
         {
            limit = (int)fmax(limit,fmin(Bars-1,_mtfCall(6,0)*TimeFrame/_Period));
            if (trend[limit]==-1) CleanPoint(limit,ADXda,ADXdb);
            for (i=limit;i>=0 && !_StopFlag; i--)
            {
                  int y = iBarShift(NULL,TimeFrame,Time[i]);
                     DIp[i]   = _mtfCall(0,y);
   	               DIm[i]   = _mtfCall(1,y);
   	               ADX[i]   = _mtfCall(2,y);
   	               ADXda[i] = EMPTY_VALUE;
                     ADXdb[i] = EMPTY_VALUE;
                     trend[i] = _mtfCall(5,y);
                     
                     //
                     //
                     //
                     //
                     //
                     
                      if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;
                      #define _interpolate(buff) buff[i+k] = buff[i]+(buff[i+n]-buff[i])*k/n
                      int n,k; datetime time = iTime(NULL,TimeFrame,y);
                         for(n = 1; (i+n)<Bars && Time[i+n] >= time; n++) continue;	
                         for(k = 1; k<n && (i+n)<Bars && (i+k)<Bars; k++) 
                         {
                            _interpolate(DIp);
                            _interpolate(DIm);
                            _interpolate(ADX);
                         }
            }
            for(i=limit; i>=0; i--) if (trend[i]==-1) PlotPoint(i,ADXda,ADXdb,ADX); 
       return(0);
       }
         
      //
      //
      //
      //
      //
   
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
      double sf = (AdxPeriod-1.0)/AdxPeriod;
      double arrc[3],arrp[2];
      if (trend[limit]==-1) CleanPoint(limit,ADXda,ADXdb);
      for (i=limit, r=Bars-i-1;i>=0;i--,r++)
      {
         if (i<Bars-1)
         {
           arrc[0] = iMA(NULL,0,Smooth,0,SmoothMethod,PriceHigh ,i  );
           arrc[1] = iMA(NULL,0,Smooth,0,SmoothMethod,PriceLow  ,i  );
           arrc[2] = iMA(NULL,0,Smooth,0,SmoothMethod,PriceClose,i+1);
           arrp[0] = iMA(NULL,0,Smooth,0,SmoothMethod,PriceHigh ,i+1);
           arrp[1] = iMA(NULL,0,Smooth,0,SmoothMethod,PriceLow  ,i+1);
           ArraySort(arrc); ArraySort(arrp); 
           double currTR  = fmax(arrc[2],arrc[1])-fmin(arrc[0],arrc[1]);
           double DeltaHi = arrc[2] - arrp[1];
	        double DeltaLo = arrp[0] - arrc[0];
           double plusDM  = 0.00;
           double minusDM = 0.00;
         
              if ((DeltaHi>DeltaLo) && (DeltaHi>0)) plusDM  = DeltaHi;
              if ((DeltaLo>DeltaHi) && (DeltaLo>0)) minusDM = DeltaLo;      
         
           //
           //
           //
           //
           //
         
              work[r][_DIp] = sf*work[r-1][_DIp] + plusDM;
              work[r][_DIm] = sf*work[r-1][_DIm] + minusDM;
              work[r][_TR]  = sf*work[r-1][_TR]  + currTR;

           //
           //
           //
           //
           //
                  
              DIp[i] = 0.00;                   
              DIm[i] = 0.00;                   
              if (work[r][_TR] > 0)             
              {              
                 DIp[i] = 100.00 * work[r][_DIp]/work[r][_TR];
                 DIm[i] = 100.00 * work[r][_DIm]/work[r][_TR];
              }            
                double DX;
                if((DIp[i] + DIm[i])>0) 
                     DX  = 100*fabs(DIp[i] - DIm[i])/(DIp[i] + DIm[i]); 
                else DX  = 0.00;
                ADX[i]   = sf*ADX[i+1] + DX/AdxPeriod; 
                ADXda[i] = EMPTY_VALUE;
                ADXdb[i] = EMPTY_VALUE;
                trend[i] = (i<Bars-1) ? (DIp[i]>DIm[i]) ? 1 : (DIp[i]<DIm[i]) ? -1 : trend[i+1] : 0;   
                if (trend[i]==-1) PlotPoint(i,ADXda,ADXdb,ADX);
            }
         }
return(0);
} 

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                          second[i] = EMPTY_VALUE; }
}

//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

   
   